@@ -11,46 +11,49 @@
 #  under the License.
 
 import abc
+import argparse
 import inspect
+import types
+import sys
+import typing as ty
 
 from stevedore import extension
 
 from cliff import _argparse
 
+if sys.version_info < (3, 10):
+    # Python 3.9 and older
+    from importlib_metadata import packages_distributions
+else:
+    # Python 3.10 and later
+    from importlib.metadata import packages_distributions  # type: ignore
+
+if ty.TYPE_CHECKING:
+    from . import app as _app
+
+_T = ty.TypeVar('_T')
 _dists_by_mods = None
 
 
-def _get_distributions_by_modules():
+def _get_distributions_by_modules() -> dict[str, str]:
     """Return dict mapping module name to distribution names.
 
     The python package name (the name used for importing) and the
     distribution name (the name used with pip and PyPI) do not
     always match. We want to report which distribution caused the
     command to be installed, so we need to look up the values.
-
     """
-    import pkg_resources
     global _dists_by_mods
     if _dists_by_mods is None:
-        results = {}
-        for dist in pkg_resources.working_set:
-            try:
-                mod_names = dist.get_metadata('top_level.txt').strip()
-            except Exception:
-                # Could not retrieve metadata. Either the file is not
-                # present or we cannot read it. Ignore the
-                # distribution.
-                pass
-            else:
-                # Distributions may include multiple top-level
-                # packages (see setuptools for an example).
-                for mod_name in mod_names.splitlines():
-                    results[mod_name] = dist.project_name
-        _dists_by_mods = results
+        # There can be multiple distribution in the case of namespace packages
+        # so we'll just grab the first one
+        _dists_by_mods = {k: v[0] for k, v in packages_distributions().items()}
     return _dists_by_mods
 
 
-def _get_distribution_for_module(module):
+def _get_distribution_for_module(
+    module: ty.Optional[types.ModuleType],
+) -> ty.Optional[str]:
     "Return the distribution containing the module."
     dist_name = None
     if module:
@@ -59,7 +62,7 @@ def _get_distribution_for_module(module):
     return dist_name
 
 
-class Command(object, metaclass=abc.ABCMeta):
+class Command(metaclass=abc.ABCMeta):
     """Base class for command plugins.
 
     When the command is instantiated, it loads extensions from a
@@ -69,28 +72,36 @@ class Command(object, metaclass=abc.ABCMeta):
         app.namespace + '.' + cmd_name.replace(' ', '_')
 
     :param app: Application instance invoking the command.
-    :paramtype app: cliff.app.App
-
+    :param app_args: Parsed arguments from options associated with the
+        application instance..
+    :param cmd_name: The name of the command.
     """
 
+    app_dist_name: ty.Optional[str]
+
     deprecated = False
     conflict_handler = 'ignore'
 
     _description = ''
     _epilog = None
 
-    def __init__(self, app, app_args, cmd_name=None):
+    def __init__(
+        self,
+        app: '_app.App',
+        app_args: ty.Optional[argparse.Namespace],
+        cmd_name: ty.Optional[str] = None,
+    ) -> None:
         self.app = app
         self.app_args = app_args
         self.cmd_name = cmd_name
         self._load_hooks()
 
-    def _load_hooks(self):
+    def _load_hooks(self) -> None:
         # Look for command extensions
-        if self.app and self.cmd_name:
+        if self.cmd_name:
             namespace = '{}.{}'.format(
                 self.app.command_manager.namespace,
-                self.cmd_name.replace(' ', '_')
+                self.cmd_name.replace(' ', '_'),
             )
             self._hooks = extension.ExtensionManager(
                 namespace=namespace,
@@ -105,7 +116,7 @@ class Command(object, metaclass=abc.ABCMeta):
             self._hooks = []
         return
 
-    def get_description(self):
+    def get_description(self) -> str:
         """Return the command description.
 
         The default is to use the first line of the class' docstring
@@ -127,31 +138,28 @@ class Command(object, metaclass=abc.ABCMeta):
             desc = ''
         return desc
 
-    def get_epilog(self):
+    def get_epilog(self) -> str:
         """Return the command epilog."""
         # replace a None in self._epilog with an empty string
         parts = [self._epilog or '']
-        hook_epilogs = filter(
-            None,
-            (h.obj.get_epilog() for h in self._hooks),
-        )
+        hook_epilogs = [
+            e for h in self._hooks if (e := h.obj.get_epilog()) is not None
+        ]
         parts.extend(hook_epilogs)
         app_dist_name = getattr(
-            self, 'app_dist_name', _get_distribution_for_module(
-                inspect.getmodule(self.app)
-            )
+            self,
+            'app_dist_name',
+            _get_distribution_for_module(inspect.getmodule(self.app)),
         )
         dist_name = _get_distribution_for_module(inspect.getmodule(self))
         if dist_name and dist_name != app_dist_name:
             parts.append(
-                'This command is provided by the %s plugin.' %
-                (dist_name,)
+                f'This command is provided by the {dist_name} plugin.'
             )
         return '\n\n'.join(parts)
 
-    def get_parser(self, prog_name):
-        """Return an :class:`argparse.ArgumentParser`.
-        """
+    def get_parser(self, prog_name: str) -> _argparse.ArgumentParser:
+        """Return an :class:`argparse.ArgumentParser`."""
         parser = _argparse.ArgumentParser(
             description=self.get_description(),
             epilog=self.get_epilog(),
@@ -164,13 +172,13 @@ class Command(object, metaclass=abc.ABCMeta):
         return parser
 
     @abc.abstractmethod
-    def take_action(self, parsed_args):
+    def take_action(self, parsed_args: argparse.Namespace) -> ty.Any:
         """Override to do something useful.
 
         The returned value will be returned by the program.
         """
 
-    def run(self, parsed_args):
+    def run(self, parsed_args: argparse.Namespace) -> int:
         """Invoked by the application when the command is run.
 
         Developers implementing commands should override
@@ -187,7 +195,9 @@ class Command(object, metaclass=abc.ABCMeta):
         return_code = self._run_after_hooks(parsed_args, return_code)
         return return_code
 
-    def _run_before_hooks(self, parsed_args):
+    def _run_before_hooks(
+        self, parsed_args: argparse.Namespace
+    ) -> argparse.Namespace:
         """Calls before() method of the hooks.
 
         This method is intended to be called from the run() method before
@@ -205,7 +215,9 @@ class Command(object, metaclass=abc.ABCMeta):
                 parsed_args = ret
         return parsed_args
 
-    def _run_after_hooks(self, parsed_args, return_code):
+    def _run_after_hooks(
+        self, parsed_args: argparse.Namespace, return_code: _T
+    ) -> _T:
         """Calls after() method of the hooks.
 
         This method is intended to be called from the run() method after
