@@ -11,10 +11,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import ctypes
 import os
-import struct
-import sys
+import typing as ty
 
 # Each edit operation is assigned different cost, such as:
 #  'w' means swap operation, the cost is 0;
@@ -25,12 +23,12 @@ import sys
 COST = {'w': 0, 's': 2, 'a': 1, 'd': 3}
 
 
-def damerau_levenshtein(s1, s2, cost):
+def damerau_levenshtein(s1: str, s2: str, cost: dict[str, int]) -> int:
     """Calculates the Damerau-Levenshtein distance between two strings.
 
-    The Levenshtein distance says the minimum number of single-character edits
-    (i.e. insertions, deletions, swap or substitution) required to change one
-    string to the other.
+    The Damerau-Levenshtein distance says the minimum number of single
+    character edits (i.e. insertions, deletions, swap or substitution)
+    required to change one string to the other.
     The idea is to reserve a matrix to hold the Levenshtein distances between
     all prefixes of the first string and all prefixes of the second, then we
     can compute the values in the matrix in a dynamic programming fashion. To
@@ -39,7 +37,7 @@ def damerau_levenshtein(s1, s2, cost):
     the row before that.)
 
     More details:
-        https://en.wikipedia.org/wiki/Levenshtein_distance
+        https://en.wikipedia.org/wiki/Damerau%E2%80%93Levenshtein_distance
         https://github.com/git/git/commit/8af84dadb142f7321ff0ce8690385e99da8ede2f
     """
 
@@ -62,7 +60,6 @@ def damerau_levenshtein(s1, s2, cost):
         row2[0] = (i + 1) * cost['d']
 
         for j in range(len2):
-
             # substitution
             sub_cost = row1[j] + (s1[i] != s2[j]) * cost['s']
 
@@ -73,11 +70,12 @@ def damerau_levenshtein(s1, s2, cost):
             del_cost = row1[j + 1] + cost['d']
 
             # swap
-            swp_condition = ((i > 0) and
-                             (j > 0) and
-                             (s1[i - 1] == s2[j]) and
-                             (s1[i] == s2[j - 1])
-                             )
+            swp_condition = (
+                (i > 0)
+                and (j > 0)
+                and (s1[i - 1] == s2[j])
+                and (s1[i] == s2[j - 1])
+            )
 
             # min cost
             if swp_condition:
@@ -93,63 +91,15 @@ def damerau_levenshtein(s1, s2, cost):
     return row1[-1]
 
 
-def terminal_width(stdout):
-    if hasattr(os, 'get_terminal_size'):
-        # python 3.3 onwards has built-in support for getting terminal size
-        try:
-            return os.get_terminal_size().columns
-        except OSError:
-            return None
+def terminal_width() -> ty.Optional[int]:
+    """Return terminal width in columns
 
-    if sys.platform == 'win32':
-        return _get_terminal_width_windows(stdout)
-    else:
-        return _get_terminal_width_ioctl(stdout)
-
-
-def _get_terminal_width_windows(stdout):
-    STD_INPUT_HANDLE = -10
-    STD_OUTPUT_HANDLE = -11
-    STD_ERROR_HANDLE = -12
-
-    std_to_win_handle = {
-        sys.stdin: STD_INPUT_HANDLE,
-        sys.stdout: STD_OUTPUT_HANDLE,
-        sys.stderr: STD_ERROR_HANDLE}
-
-    std_handle = std_to_win_handle.get(stdout)
-    if not std_handle:
-        return None
-
-    handle = ctypes.windll.kernel32.GetStdHandle(std_handle)
-    csbi = ctypes.create_string_buffer(22)
-
-    res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(handle, csbi)
-    if res:
-        (size_x, size_y, cur_pos_x, cur_pos_y, attr,
-         left, top, right, bottom, max_size_x, max_size_y) = struct.unpack(
-            "hhhhHhhhhhh", csbi.raw)
-        return size_x
-
-
-def _get_terminal_width_ioctl(stdout):
-    from fcntl import ioctl
-    import termios
+    Uses `os.get_terminal_size` function
 
+    :returns: terminal width
+    :rtype: int or None
+    """
     try:
-        # winsize structure has 4 unsigned short fields
-        winsize = b'\0' * struct.calcsize('hhhh')
-        try:
-            winsize = ioctl(stdout, termios.TIOCGWINSZ, winsize)
-        except IOError:
-            return None
-        except TypeError:
-            # this is raised in unit tests as stdout is sometimes a StringIO
-            return None
-        winsize = struct.unpack('hhhh', winsize)
-        columns = winsize[1]
-        if not columns:
-            return None
-        return columns
-    except IOError:
+        return os.get_terminal_size().columns
+    except OSError:
         return None
